
#import <RWKit/RWKit.h>



@class YHMainViewController;
@class YHWebControlView;
@class YHPopoverController;
@class YHTemplatesPopoverController;
@class YHMediaPopoverController;
@class YHPopoverWindow;
@class YHInfoController;
@class YHGlobalController;
@class YHSlide;
@class YHIndexSet;
@class YHToolbarView;
@class YHNavbarView;
@class YHSandwich;

@class RMSandwich;
@class RMSandwichFilling;



@protocol YHStacksImageSaving;



enum {
    kYHViewModeNil,
    kYHViewModeTall,
    kYHViewModeGrande,
    kYHViewModeVenti
};
typedef NSUInteger YHViewMode;



@interface YHStacksKitPlugin : RWTextViewPluginClient <RWLinkPanelDelegate, NSTextViewDelegate>
{
    NSNib                   *_nib;
    NSArray                 *_topObjects;
    
    NSView                  *_stacksView;
	NSView                  *_mainWrapperView;
    YHWebControlView		*_controlView;
    WebView                 *_webView;
    NSView                  *_sideView;
    YHSlide                 *_viewModeSlide;
    YHToolbarView           *_toolbarView;
    YHNavbarView			*_navbarView;
    NSView                  *_optionsView;

    NSButton                *_stacksPopoverButton;
    NSButton                *_templatesPopoverButton;
    NSButton                *_mediaPopoverButton;

    YHMainViewController    *_nodeController;
	YHInfoController        *_infoController;
    YHGlobalController      *_globalController;

    NSString                *_guid;
    NSString                *_contentHTMLString;
    NSDictionary            *_exportParams;
    RWExportParameters      *_eParams;
    NSString                *_tempFilesDirectory;
    
    NSString                *_consoleString;

    NSWindow                *_devWindow;
    NSTextView              *_debugDumpView;
    NSTextView              *_debugLogView;
	NSMutableString         *_debugLogBuffer;
    
    BOOL                    _valid;
    BOOL                    _valid1;
    BOOL                    _validUp;
}

#pragma mark - Properties
@property (retain)  IBOutlet    NSView                  *stacksView;
@property (retain)  IBOutlet    NSView                  *mainWrapperView;
@property (retain)  IBOutlet    YHWebControlView        *controlView;
@property (retain)  IBOutlet    WebView                 *webView;
@property (retain)  IBOutlet    NSView                  *sideView;
@property (retain)  IBOutlet    YHSlide                 *viewModeSlide;
@property (retain)  IBOutlet    YHToolbarView           *toolbarView;
@property (retain)  IBOutlet    YHNavbarView			*navbarView;
@property (retain)  IBOutlet    NSView                  *optionsView;

@property (retain)  IBOutlet    YHMainViewController    *nodeController;
@property (retain)              YHInfoController        *infoController;
@property (retain)              YHGlobalController      *globalController;

//@property (readonly)            YHPopoverWindow         *stacksPopoverWindow;
//@property (readonly)            YHPopoverController     *stacksPopoverController;

@property (assign)      YHViewMode          viewMode;

@property (readonly)    NSString            *pageID;
@property (readonly)    id                  rwPage;
@property (readonly)    NSString            *guid;
@property (readonly)    NSUndoManager       *sharedUndoManager;
@property (readonly)    YHIndexSet          *nidCache;
@property (retain)      NSString            *contentHTMLString;
@property (retain)      NSDictionary        *exportParams;
@property (retain)      RWExportParameters  *eParams;

@property (readonly)    NSString            *cssFileName;
@property (readonly)    NSString            *phpFileName;
@property (readonly)    NSString            *jsFileName;
@property (readonly)    NSString            *tempFilesDirectory;
@property (readonly)    NSString            *pagePath;
@property (readonly)    NSString            *filesFolderName;
@property (readonly)    NSString            *commonFilesFolderName;
@property (readonly)    NSString            *relativeCommonFilesFolderName;
@property (readonly)    NSString            *relativeDocRoot;
@property (readonly)    NSString            *assetsFolderName;
@property (readonly)    NSString            *versionString;

@property (assign)      BOOL                valid;
@property (assign)      BOOL                valid1;
@property (assign)      BOOL                validUp;

@property (nonatomic, strong) NSNib                 *nib;
@property (nonatomic, strong) NSArray               *topObjects;


- (void)updateViewMode;



#pragma mark - RWPlugin


+ (YHPopoverWindow *)stacksPopoverWindow;
+ (YHPopoverController *)stacksPopoverController;


#pragma mark - Popovers and Sideview
- (IBAction)doCloseAllPopovers:(id)sender;
- (IBAction)doPopoverStacksLibrary:(id)sender;
- (IBAction)doPopoverTemplatesLibrary:(id)sender;
- (IBAction)doPopoverMediaLibrary:(id)sender;

- (void)checkPopoverFrames;

- (IBAction)doToggleSideView:(id)sender;
- (void)refreshInfo;
- (void)reloadStackLibrary;



#pragma mark - Pages Controller
+ (id)mainPage;
+ (id)pageForWindow:(NSWindow *)window;
+ (void)addPage:(id)page;
+ (void)removePage:(id)page;
+ (NSArray *)allPages;

- (id)stacksDocument;
- (id)stacksDocumentWindow;



#pragma mark - Publishing
+ (NSString *)previewCSSPath;
+ (NSString *)publishCSSPath;
+ (NSString *)publishIECSSPath;
+ (NSString *)publishCloudPhpPath;
- (void)createFilesFolder;
- (void)exportAsset:(NSString *)source toDestination:(NSString *)destination;



#pragma mark - Developer Tools
//+ (NSMutableString *)sharedConsoleString;
//+ (void)appendToConsole:(NSString *)string;
//+ (void)prependPreviousConsole:(NSString *)string;
//+ (void)doConsoleInfo:(NSString *)string;
//+ (void)doConsoleWarning:(NSString *)string;
//+ (void)doConsoleError:(NSString *)string;
+ (void)reportFatalErrorWithSandwichLog:(NSString *)log message:(NSString *)message;




#pragma mark - Developer Tools
- (IBAction)doDebugDumpPreview:(id)sender;
- (IBAction)doDebugDumpPlist:(id)sender;

+ (void)doDevLog:(NSString *)logString;
- (void)doDevLog:(NSString *)logString;
- (IBAction)doOpenDevWindow:(id)sender;
- (IBAction)doDevReloadLibrary:(id)sender;
- (IBAction)doDevConnect:(id)sender;
- (IBAction)doDevLogClear:(id)sender;
- (IBAction)doDevInfoDump:(id)sender;



#pragma mark - Serial Validation
- (IBAction)doBuy:(id)sender;
- (IBAction)doWebStore:(id)sender;
- (IBAction)doWebUpgradeStore:(id)sender;
- (IBAction)doHelp:(id)sender;
- (BOOL)validateAddition;



@end



#pragma mark - YHStacksImageSaving



@protocol YHStacksImageSaving

@property (retain) YHSandwich *yhSandwich;

- (void)addImage:(NSURL *)url forKey:(NSString *)key;
- (NSString *)addImage:(NSURL *)url;
- (NSURL *)addCustomImage:(NSURL *)url;

- (NSURL *)urlForFilename:(NSString *)fileName;
- (NSURL *)urlForUrl:(NSURL *)oldUrl;

@end



#pragma mark - RMSandwichFilling (NoDeleting)



@interface RMSandwichFilling (NoDeleting)

//- (BOOL)checkIntegrity;

@end



#pragma mark - RMSandwich (NoDeleting)



@interface RMSandwich (NoDeleting)

@property   (retain)    RMSandwichFilling   *filling;
@property   (copy)      NSDictionary        *dictionary;
@property   (copy)      NSDictionary        *files;
@property   (copy)      NSDictionary        *subsandwiches;
@property   (readonly)  NSMutableDictionary *mutableDictionary;
@property   (readonly)  NSMutableDictionary *mutableFiles;
@property   (readonly)  NSMutableDictionary *mutableSubsandwiches;

//- (BOOL)checkIntegrity;
- (void)setDictionaryObject:(id)object forKey:(NSString *)key;
- (void)setSubsandwich:(id)object forKey:(NSString *)key;
- (void)setFile:(id)object forKey:(NSString *)key;

@end



#pragma mark - Privateer



@interface RMHTML (YHStacksKitPlugin)

- (void)processLink:(id)link 
			   page:(NSObject*)thePage
	depthCorrection:(int)depthCorrection 
			 toHTML:(NSMutableString*)html
	filesFolderName:(NSString*)filesFolderName;

@end
